@@ -11,27 +11,25 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.827';
+our $VERSION = '1.828';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
 our ($display_die_with_localtime, $display_warn_with_localtime);
 our ($main_proc_id, $prog_name, $tmp_dir);
 
-my  ($_is_winenv);
+tie $tmp_dir, 'MCE::Signal::_tmpdir';
 
 use Carp ();
 
 BEGIN {
-   local $@; local $SIG{__DIE__};
+   local $@;
 
    $main_proc_id =  $$;
    $prog_name    =  $0;
    $prog_name    =~ s{^.*[\\/]}{}g;
    $prog_name    =  'perl' if ($prog_name eq '-e' || $prog_name eq '-');
 
-   $_is_winenv   =  ($^O =~ /mswin|mingw|msys|cygwin/i) ? 1 : 0;
-
    if ($^O eq 'MSWin32' && !$INC{'threads.pm'}) {
       eval 'use threads; use threads::shared';
    }
@@ -42,10 +40,8 @@ BEGIN {
    return;
 }
 
-use File::Path ();
-use Time::HiRes qw( sleep time );
-use Fcntl qw( :flock );
 use base qw( Exporter );
+use Time::HiRes ();
 
 our @EXPORT_OK = qw( $tmp_dir sys_cmd stop_and_exit );
 our %EXPORT_TAGS = (
@@ -55,6 +51,18 @@ our %EXPORT_TAGS = (
 
 sub _NOOP {}
 
+END {
+   if ($$ == $main_proc_id) {
+      if (defined $MCE::Signal::SIGNAME) {
+         $SIG{ $MCE::Signal::SIGNAME } = 'DEFAULT';
+         CORE::kill($MCE::Signal::SIGNAME, $$);
+      }
+      else {
+         MCE::Signal::stop_and_exit($?);
+      }
+   }
+}
+
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Process import, export, & module arguments.
@@ -67,31 +75,28 @@ sub _flag  { 1 }
 
 my $_is_MSWin32   = ($^O eq 'MSWin32') ? 1 : 0;
 my $_keep_tmp_dir = 0;
-my $_no_sigmsg    = 0;
+my $_use_dev_shm  = 0;
 my $_no_kill9     = 0;
 my $_imported;
 
 sub import {
+   my $_class = shift;
+   return if $_imported++;
 
-   my $_class = shift; return if ($_imported++);
-
-   my @_export_args = ();
-   my $_no_setpgrp  = 0;
-   my $_setpgrp     = 0;
-   my $_use_dev_shm = 0;
+   my ($_no_setpgrp, $_no_sigmsg, $_setpgrp, @_export_args) = (0, 0, 0);
 
    while (my $_arg = shift) {
-
+      $_setpgrp      = _flag() and next if ($_arg eq '-setpgrp');
       $_keep_tmp_dir = _flag() and next if ($_arg eq '-keep_tmp_dir');
-      $_no_sigmsg    = _flag() and next if ($_arg eq '-no_sigmsg');
+      $_use_dev_shm  = _flag() and next if ($_arg eq '-use_dev_shm');
       $_no_kill9     = _flag() and next if ($_arg eq '-no_kill9');
 
+      # deprecated options for backwards compatibility
       $_no_setpgrp   = _flag() and next if ($_arg eq '-no_setpgrp');
-      $_setpgrp      = _flag() and next if ($_arg eq '-setpgrp');
-
-      $_use_dev_shm  = _flag() and next if ($_arg eq '-use_dev_shm');
+      $_no_sigmsg    = _flag() and next if ($_arg eq '-no_sigmsg');
 
       _usage($_arg) if ($_arg =~ /^-/);
+
       push @_export_args, $_arg;
    }
 
@@ -101,31 +106,8 @@ sub import {
    ## Sets the current process group for the current process.
    setpgrp(0,0) if ($_setpgrp == 1 && !$_is_MSWin32);
 
-   my ($_count, $_tmp_base_dir) = (0);
-
-   if ($ENV{TEMP} && -d $ENV{TEMP} && -w $ENV{TEMP}) {
-      if ( $_is_winenv ) {
-         $_tmp_base_dir = $ENV{TEMP} . '/Perl-MCE';
-         mkdir $_tmp_base_dir unless (-d $_tmp_base_dir);
-      }
-      else {
-         $_tmp_base_dir = $ENV{TEMP};
-      }
-   }
-   else {
-      $_tmp_base_dir = ($_use_dev_shm && -d '/dev/shm' && -w '/dev/shm')
-         ? '/dev/shm' : '/tmp';
-   }
-
-   _croak("Error: MCE::Signal: ($_tmp_base_dir) is not writeable")
-      if (! exists $ENV{'MOBASTARTUPDIR'} && ! -w $_tmp_base_dir);
-
-   ## Remove tainted'ness from $tmp_dir.
-   ($tmp_dir) = "$_tmp_base_dir/$prog_name.$$.$_count" =~ /(.*)/;
-
-   while ( !(mkdir $tmp_dir, 0770) ) {
-      ($tmp_dir) = ("$_tmp_base_dir/$prog_name.$$.".(++$_count)) =~ /(.*)/;
-   }
+   ## Make tmp_dir if caller requested it.
+   _make_tmpdir() if (grep /tmp_dir/, @_export_args);
 
    return;
 }
@@ -146,9 +128,154 @@ $SIG{TERM} = \&stop_and_exit;  # UNIX SIG 15
 ## MCE handles the reaping of its children.
 $SIG{CHLD} = 'DEFAULT' unless $_is_MSWin32;
 
-## Call stop_and_exit when exiting the script.
-END {
-   MCE::Signal->stop_and_exit($?) if ($$ == $main_proc_id);
+my $_safe_clean = 0;
+
+sub _make_tmpdir {
+   my ($_count, $_tmp_base_dir) = (0);
+
+   return $tmp_dir if (defined $tmp_dir && -d $tmp_dir && -w _);
+
+   if ($ENV{TEMP} && -d $ENV{TEMP} && -w _) {
+      if ($^O =~ /mswin|mingw|msys|cygwin/i) {
+         $_tmp_base_dir = $ENV{TEMP} . '/Perl-MCE';
+         mkdir $_tmp_base_dir unless -d $_tmp_base_dir;
+      } else {
+         $_tmp_base_dir = $ENV{TEMP};
+      }
+   }
+   else {
+      $_tmp_base_dir = ($_use_dev_shm && -d '/dev/shm' && -w _)
+         ? '/dev/shm' : '/tmp';
+   }
+
+   _croak("Error: MCE::Signal: ($_tmp_base_dir) is not writeable")
+      if (! exists $ENV{'MOBASTARTUPDIR'} && ! -w $_tmp_base_dir);
+
+   ## Remove tainted'ness from $tmp_dir.
+   ($tmp_dir) = "$_tmp_base_dir/$prog_name.$$.$_count" =~ /(.*)/;
+
+   while ( !(mkdir $tmp_dir, 0770) ) {
+      ($tmp_dir) = ("$_tmp_base_dir/$prog_name.$$.".(++$_count)) =~ /(.*)/;
+   }
+
+   $_safe_clean = 1;
+
+   return $tmp_dir;
+}
+
+sub _remove_tmpdir {
+   return if (!defined $tmp_dir || $tmp_dir eq '' || ! -d $tmp_dir);
+
+   if ($_keep_tmp_dir == 1) {
+      print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
+   }
+   elsif ($_safe_clean) {
+      if ($ENV{'TEMP'} && $^O =~ /mswin|mingw|msys|cygwin/i) {
+         ## remove tainted'ness
+         my ($_dir) = $ENV{'TEMP'} =~ /(.*)/;
+         chdir $_dir if -d $_dir;
+      }
+      rmdir $tmp_dir;
+      if (-d $tmp_dir) {
+         local $@; local $SIG{__DIE__};
+         eval 'require File::Path; File::Path::rmtree($tmp_dir)';
+      }
+   }
+
+   $tmp_dir = undef;
+}
+
+###############################################################################
+## ----------------------------------------------------------------------------
+## Stops execution, removes temp directory and exits cleanly.
+##
+## Provides safe reentrant logic for parent and child processes.
+## The $main_proc_id variable is defined above.
+##
+###############################################################################
+
+my %_sig_name_lkup = map { $_ => 1 } qw(
+   CHLD __DIE__ HUP INT PIPE QUIT TERM __WARN__
+);
+
+my $_handler_cnt : shared = 0;
+
+sub stop_and_exit {
+   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
+
+   my ($_exit_status, $_is_sig, $_sig_name) = ($?, 0, $_[0] || 0);
+   $SIG{__DIE__} = $SIG{__WARN__} = \&_NOOP;
+
+   if (!defined $MCE::Signal::STATUS) {
+      if (!exists $_sig_name_lkup{ $_sig_name }) {
+         $MCE::Signal::STATUS = $_sig_name if ($_sig_name =~ /^\d+$/);
+      } else {
+         $SIG{INT} = $SIG{$_sig_name} = \&_NOOP;
+         $MCE::Signal::STATUS = $_is_sig = $MCE::Signal::KILLED = 1;
+         $MCE::Signal::STATUS = 255 if ($_sig_name eq '__DIE__');
+         $MCE::Signal::STATUS = 0 if ($_sig_name eq 'PIPE');
+      }
+   }
+
+   ## Main process.
+   if ($$ == $main_proc_id) {
+      lock $_handler_cnt if $INC{'threads/shared.pm'};
+
+      if (++$_handler_cnt == 1) {
+         ## Kill process group if signaled.
+         if ($_is_sig == 1) {
+            if ($_sig_name eq 'INT' || $_sig_name eq 'TERM') {
+               $MCE::Signal::SIGNAME = $_sig_name;
+            }
+            ($_sig_name eq 'PIPE')
+               ? CORE::kill('PIPE', $_is_MSWin32 ? -$$ : -getpgrp)
+               : CORE::kill('INT' , $_is_MSWin32 ? -$$ : -getpgrp);
+
+            if ($_sig_name eq 'PIPE') {
+               for my $_i (1..2) { Time::HiRes::sleep(0.015); }
+            } else {
+               for my $_i (1..3) { Time::HiRes::sleep(0.060); }
+            }
+         }
+
+         ## Remove temp directory.
+         _remove_tmpdir() if defined($tmp_dir);
+
+         ## Signal process group to die.
+         if ($_is_sig == 1) {
+            if ($INC{'threads.pm'} && ($] lt '5.012000' || threads->tid())) {
+               ($_no_kill9 == 1 || $_sig_name eq 'PIPE')
+                  ? CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp)
+                  : CORE::kill('KILL', -$$, $main_proc_id);
+            }
+            else {
+               CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp);
+            }
+         }
+      }
+   }
+
+   ## Child processes.
+   elsif ($_is_sig) {
+      lock $_handler_cnt if $INC{'threads/shared.pm'};
+
+      ## Windows support, from nested workers.
+      if ($_is_MSWin32) {
+         _remove_tmpdir() if defined($tmp_dir);
+         CORE::kill('KILL', $main_proc_id, -$$);
+      }
+
+      ## Real child processes.
+      else {
+         CORE::kill($_sig_name, $main_proc_id, -$$);
+         CORE::kill('KILL', $$, -$$);
+      }
+   }
+
+   ## Exit with status.
+   ( defined $MCE::Signal::STATUS )
+      ? CORE::exit( $MCE::Signal::STATUS )
+      : CORE::exit( $_exit_status );
 }
 
 ###############################################################################
@@ -166,7 +293,6 @@ END {
 ###############################################################################
 
 sub sys_cmd {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    _croak('MCE::Signal::sys_cmd: no arguments were specified') if (@_ == 0);
@@ -186,172 +312,6 @@ sub sys_cmd {
    return $_exit_status;
 }
 
-###############################################################################
-## ----------------------------------------------------------------------------
-## Stops execution, removes temp directory and exits cleanly.
-##
-## Provides safe reentrant logic for both parent and child processes.
-## The $main_proc_id variable is defined above.
-##
-###############################################################################
-
-my %_sig_name_lkup = map { $_ => 1 } qw(
-   __DIE__ __WARN__ HUP INT PIPE QUIT TERM CHLD
-);
-
-my $_handler_cnt : shared = 0;
-
-sub stop_and_exit {
-
-   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
-
-   my $_is_main_proc = ($$ == $main_proc_id);
-   my $_sig_name     = $_[0] || 0;
-   my $_exit_status  = $?;
-   my $_is_sig       = 0;
-
-   if (exists $_sig_name_lkup{$_sig_name}) {
-      $_exit_status     = $MCE::Signal::KILLED = $_is_sig = 1;
-      $SIG{$_sig_name}  = \&_NOOP;
-   }
-   else {
-      $_exit_status = $_sig_name if ($_sig_name =~ /^\d+$/);
-   }
-
-   $SIG{INT} = $SIG{__DIE__} = $SIG{__WARN__} = \&_NOOP;
-
-   ## -------------------------------------------------------------------------
-
-   ## For the main process.
-   if ($_is_main_proc) {
-      lock $_handler_cnt if $INC{'threads/shared.pm'};
-
-      if (++$_handler_cnt == 1) {
-         open my $_FH, '>>', "$tmp_dir/stopped";
-         close   $_FH;
-
-         local $\ = undef;
-
-         ## Display message and kill process group if signaled.
-         if ($_is_sig == 1) {
-            my $_err_msg = undef;
-
-            if ($_sig_name eq 'INT' && -f "$tmp_dir/died") {
-               $_err_msg  = 'caught signal (__DIE__), exiting';
-            }
-            elsif ($_sig_name eq '__DIE__') {
-               $_err_msg  = 'caught signal (__DIE__), exiting';
-               $_sig_name = 'INT';
-            }
-            elsif ($_sig_name ne 'PIPE') {
-               $_err_msg  = "caught signal ($_sig_name), exiting";
-            }
-
-            if ($_err_msg && $_no_sigmsg == 0) {
-               print {*STDERR} "\n## $prog_name: $_err_msg\n";
-            }
-
-            open my $_FH, '>', "$tmp_dir/killed";
-            close   $_FH;
-
-            CORE::kill($_sig_name, $_is_MSWin32 ? -$$ : -getpgrp);
-
-            ## Pause a bit.
-            if ($_sig_name eq 'PIPE') {
-               sleep 0.015 for (1..2);
-            } else {
-               sleep 0.060 for (1..3);
-            }
-         }
-
-         ## Remove temp directory.
-         if (defined $tmp_dir && $tmp_dir ne '' && -d $tmp_dir) {
-            if ($_keep_tmp_dir == 1) {
-               print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
-            }
-            elsif ($tmp_dir ne '/tmp' && $tmp_dir ne '/var/tmp') {
-               if ($_is_winenv && $ENV{'TEMP'}) {
-                  ## remove tainted'ness
-                  my ($_t) = $ENV{'TEMP'} =~ /(.*)/;
-                  chdir $_t;
-               }
-               eval q{ File::Path::rmtree($tmp_dir) };
-            }
-            $tmp_dir = undef;
-         }
-
-         ## Signal process group to die.
-         if ($_is_sig == 1) {
-            if ($_sig_name eq 'PIPE' && $INC{'MCE/Hobo.pm'}) {
-               CORE::kill('QUIT', $_is_MSWin32 ? -$$ : -getpgrp);
-            }
-            if ($_sig_name ne 'PIPE' && $_no_sigmsg == 0) {
-               print {*STDERR} "\n";
-            }
-            ($_no_kill9 == 1 || $_sig_name eq 'PIPE')
-               ? CORE::kill('INT', $_is_MSWin32 ? -$$ : -getpgrp)
-               : CORE::kill('KILL', -$$, $main_proc_id);
-         }
-      }
-   }
-
-   ## Otherwise, for child processes.
-   elsif ($_is_sig == 1 && $tmp_dir && -d $tmp_dir) {
-      lock $_handler_cnt if $INC{'threads/shared.pm'};
-
-      if (++$_handler_cnt == 1) {
-         if ($^O eq 'MSWin32') {
-            if ($_keep_tmp_dir == 1) {
-               print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
-            }
-            elsif ($tmp_dir ne '/tmp' && $tmp_dir ne '/var/tmp') {
-               chdir $ENV{'TEMP'} if ($_is_winenv && $ENV{'TEMP'});
-               eval q{ File::Path::rmtree($tmp_dir) };
-            }
-            $tmp_dir = undef;
-            CORE::kill('KILL', $main_proc_id, -$$);
-         }
-         else {
-            ## Obtain lock.
-            open my $CHILD_LOCK, '+>>', "$tmp_dir/child.lock";
-            flock $CHILD_LOCK, LOCK_EX;
-
-            ## Notify the main process that I've died.
-            if ($_sig_name eq '__DIE__' && ! -f "$tmp_dir/died") {
-               local $@; eval {
-                  open my $_FH, '>>', "$tmp_dir/died";
-                  close   $_FH;
-               };
-            }
-
-            ## Signal process group to terminate.
-            if (! -f "$tmp_dir/killed" && ! -f "$tmp_dir/stopped") {
-               local $@; eval {
-                  open my $_FH, '>>', "$tmp_dir/killed";
-                  close   $_FH;
-               };
-               ($_sig_name eq 'PIPE')
-                  ? CORE::kill('PIPE', $main_proc_id, -$$)
-                  : CORE::kill('INT', $main_proc_id, -$$);
-            }
-
-            ## Release lock.
-            flock $CHILD_LOCK, LOCK_UN;
-            close $CHILD_LOCK;
-         }
-      }
-   }
-
-   ## -------------------------------------------------------------------------
-
-   ## Exit with status.
-   if ($_is_sig == 1 && $_no_kill9 == 0) {
-      sleep 0.060 for (1..5);
-   }
-
-   CORE::exit($_exit_status);
-}
-
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Signal handlers for __DIE__ & __WARN__ utilized by MCE.
@@ -359,7 +319,6 @@ sub stop_and_exit {
 ###############################################################################
 
 sub _die_handler {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    if (!defined $^S || $^S) {
@@ -395,11 +354,10 @@ sub _die_handler {
       }
    }
 
-   MCE::Signal->stop_and_exit('__DIE__');
+   MCE::Signal::stop_and_exit('__DIE__');
 }
 
 sub _warn_handler {
-
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
    ## Ignore thread warnings during exiting.
@@ -431,6 +389,46 @@ sub _warn_handler {
 
 1;
 
+###############################################################################
+## ----------------------------------------------------------------------------
+## TIE scalar package for making $MCE::Signal::tmp_dir on demand.
+##
+###############################################################################
+
+package MCE::Signal::_tmpdir;
+
+sub TIESCALAR {
+   my $_class = shift;
+   bless \do{ my $o = defined $_[0] ? shift : undef }, $_class;
+}
+
+sub STORE {
+   ${ $_[0] } = $_[1];
+
+   $_safe_clean = 0 if ( length $_[1] < 9 );
+   $_safe_clean = 0 if ( $ENV{'TEMP'} && $ENV{'TEMP'} eq $_[1] );
+   $_safe_clean = 0 if ( $_[1] =~ m{[\\/](?:etc|bin|lib|sbin)} );
+   $_safe_clean = 0 if ( $_[1] =~ m{[\\/](?:temp|tmp)[\\/]?$}i );
+
+   $_[1];
+}
+
+sub FETCH {
+   if (!defined ${ $_[0] }) {
+      my $_caller = caller();
+      if ($_caller ne 'MCE' && $_caller ne 'MCE::Signal') {
+         if ($INC{'MCE.pm'} && MCE->wid() > 0) {
+            ${ $_[0] } = MCE->tmp_dir();
+         } else {
+            ${ $_[0] } = MCE::Signal::_make_tmpdir();
+         }
+      }
+   }
+   ${ $_[0] };
+}
+
+1;
+
 __END__
 
 ###############################################################################
@@ -445,7 +443,7 @@ MCE::Signal - Temporary directory creation/cleanup and signal handling
 
 =head1 VERSION
 
-This document describes MCE::Signal version 1.827
+This document describes MCE::Signal version 1.828
 
 =head1 SYNOPSIS
 
@@ -496,7 +494,6 @@ The following are available options and their meanings.
 
  -use_dev_shm      - Create the temporary directory under /dev/shm
 
- -no_sigmsg        - Do not display a message when receiving a signal
  -no_kill9         - Do not kill -9 after receiving a signal to terminate
 
  -setpgrp          - Calls setpgrp to set the process group for the process
